/*
 *   Copyright (c) 2009, Michael Lehn
 *
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1) Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2) Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *   3) Neither the name of the FLENS development group nor the names of
 *      its contributors may be used to endorse or promote products derived
 *      from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <cxxblas/aux/aux.h>

namespace cxxblas {

template <typename IndexType, typename X, typename T>
void
nrm2(IndexType n, const X *x, IndexType incX, T &norm)
{
    norm = 0;
    for (IndexType i=0; i<n; ++i, x+=incX) {
        norm += conjugate(*x)*(*x);
    }
    norm = std::sqrt(norm);
}

#ifdef HAVE_CBLAS

// snrm2
template <typename IndexType>
typename If<IndexType>::isBlasCompatibleInteger
nrm2(IndexType n, const float *x, IndexType incX, float &norm)
{
    CXXBLAS_DEBUG_OUT("[" BLAS_IMPL "] cblas_snrm2");

    norm = cblas_snrm2(n, x, incX);
}

// dnrm2
template <typename IndexType>
typename If<IndexType>::isBlasCompatibleInteger
nrm2(IndexType n, const double *x, IndexType incX, double &norm)
{
    CXXBLAS_DEBUG_OUT("[" BLAS_IMPL "] cblas_dnrm2");

    norm = cblas_dnrm2(n, x, incX);
}

// scnrm2
template <typename IndexType>
typename If<IndexType>::isBlasCompatibleInteger
nrm2(IndexType n, const ComplexFloat *x, IndexType incX, float &norm)
{
    CXXBLAS_DEBUG_OUT("[" BLAS_IMPL "] cblas_scnrm2");

    norm = cblas_scnrm2(n, x, incX);
}

// dznrm2
template <typename IndexType>
typename If<IndexType>::isBlasCompatibleInteger
nrm2(IndexType n, const ComplexDouble *x, IndexType incX, double &norm)
{
    CXXBLAS_DEBUG_OUT("[" BLAS_IMPL "] cblas_dznrm2");

    norm = cblas_dznrm2(n, x, incX);
}

#endif // HAVE_CBLAS

} // namespace cxxblas
