/*
 *   Copyright (c) 2009, Michael Lehn
 *
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1) Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2) Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *   3) Neither the name of the FLENS development group nor the names of
 *      its contributors may be used to endorse or promote products derived
 *      from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <complex>
#include <cxxblas/level1/level1.h>

namespace cxxblas {

template <typename IndexType, typename ALPHA, typename MA, typename VX,
          typename VY>
void
ger_generic(StorageOrder order,
            IndexType m, IndexType n,
            const ALPHA &alpha,
            const VX *x, IndexType incX,
            const VY *y, IndexType incY,
            MA *A, IndexType ldA)
{
/*    if (order==ColMajor) {
        trans = Transpose(trans^Trans);
        gemv_generic(RowMajor, trans, n, m, alpha, A, ldA,
                     x, incX, beta, y, incY);
        return;
    }
    if ((trans==NoTrans) || (trans==Conj)) {
        scal(m, beta, y, incY);
        if (trans==Conj) {
            for (IndexType i=0, iY=0; i<m; ++i, iY+=incY) {
                VY _y;
                dot(n, A+i*ldA, IndexType(1), x, incX, _y);
                y[iY] += alpha*_y;
            }
        } else {
            for (IndexType i=0, iY=0; i<m; ++i, iY+=incY) {
                VY _y;
                dotu(n, A+i*ldA, IndexType(1), x, incX, _y);
                y[iY] += alpha*_y;
            }
        }
    } else {
        scal(n, beta, y, incY);
        if (trans==ConjTrans) {
            for (IndexType i=0, iY=0; i<n; ++i, iY+=incY) {
                VY _y;
                dot(n, A+i, ldA, x, incX, _y);
                y[iY] += alpha*_y;
            }
        } else {
            for (IndexType i=0, iY=0; i<n; ++i, iY+=incY) {
                VY _y;
                dotu(n, A+i, ldA, x, incX, _y);
                y[iY] += alpha*_y;
            }
        }
    }
*/}

//------------------------------------------------------------------------------

template <typename IndexType, typename ALPHA, typename MA, typename VX,
          typename VY>
void
ger(StorageOrder order,
    IndexType m, IndexType n,
    const ALPHA &alpha,
    const VX *x, IndexType incX,
    const VY *y, IndexType incY,
    MA *A, IndexType ldA)
{
    ger_generic(order, m, n, alpha, x, incX, y, incY, A, ldA);
}

#ifdef HAVE_CBLAS

// sger
template <typename IndexType>
typename If<IndexType>::isBlasCompatibleInteger
ger(StorageOrder order,
    IndexType m, IndexType n,
    float alpha,
    const float *x, IndexType incX,
    const float *y, IndexType incY,
    float *A, IndexType ldA)
{
    CXXBLAS_DEBUG_OUT("[" BLAS_IMPL "] cblas_sger");

    cblas_sger(CBLAS::value(order),
               m,  n,
               alpha,
               x, incX,
               y, incY,
               A, ldA);
}

// dger
template <typename IndexType>
typename If<IndexType>::isBlasCompatibleInteger
ger(StorageOrder order,
    IndexType m, IndexType n,
    double alpha,
    const double *x, IndexType incX,
    const double *y, IndexType incY,
    double *A, IndexType ldA)
{
    CXXBLAS_DEBUG_OUT("[" BLAS_IMPL "] cblas_dger");

    cblas_dger(CBLAS::value(order),
                m,  n,
                alpha,
                x, incX,
                y, incY,
                A, ldA);
}

// cger
template <typename IndexType>
typename If<IndexType>::isBlasCompatibleInteger
ger(StorageOrder order, Transpose trans,
    IndexType m, IndexType n,
    ComplexFloat alpha,
    const ComplexFloat *A, IndexType ldA,
    const ComplexFloat *x, IndexType incX,
    ComplexFloat beta,
    ComplexFloat *y, IndexType incY)
{
    CXXBLAS_DEBUG_OUT("[" BLAS_IMPL "] cblas_cger");

    cblas_cger(CBLAS::value(order), CBLAS::value(trans),
               m,  n,
               reinterpret_cast<const float *>(alpha),
               reinterpret_cast<const float *>(x), incX,
               reinterpret_cast<const float *>(y), incY,
               reinterpret_cast<const float *>(A), ldA);
}

// zger
template <typename IndexType>
typename If<IndexType>::isBlasCompatibleInteger
ger(StorageOrder order, Transpose trans,
    IndexType m, IndexType n,
    ComplexDouble alpha,
    const ComplexDouble *A, IndexType ldA,
    const ComplexDouble *x, IndexType incX,
    ComplexDouble beta,
    ComplexDouble *y, IndexType incY)
{
    CXXBLAS_DEBUG_OUT("[" BLAS_IMPL "] cblas_zger");

    cblas_zger(CBLAS::value(order), CBLAS::value(trans),
               m,  n,
               reinterpret_cast<const double *>(alpha),
               reinterpret_cast<const double *>(x), incX,
               reinterpret_cast<const double *>(y), incY,
               reinterpret_cast<const double *>(A), ldA);
}

#endif // HAVE_CBLAS

} // namespace cxxblas
